#!/bin/bash
IFS=$'\n\t'

# -e: immediately exit if any command has a non-zero exit status
# -o: prevents errors in a pipeline from being masked
# IFS new value is less likely to cause confusing bugs when looping arrays or arguments (e.g. $@)

usage() { echo "Usage: $0 -b <subscriptionId> -g <resourceGroupName> -l <resourceGroupLocation> -i <feedbotId> -n <feedbotName> -o <originUrl> -a <appId> -s <appSecret>" 1>&2; exit 1; }

declare subscriptionId="05de1293-e942-45b4-bce2-b8dd3ecac227"
declare resourceGroupLocation="West Europe"
declare deploymentName="init"
declare resourceGroupName=""
declare feedbotName="Chatbot"
declare feedbotId=""
declare appId=""
declare appSecret=""
declare origin=""
declare botLocale="cs"
declare callbackUrl=""
declare callbackToken=""

# Check needed secret variables
if [[ ! -n "${FUNCTION_HOST_KEY_APP_SECRET}" ]]; then
  echo 'Env variable FUNCTION_HOST_KEY_APP_SECRET must be set to secret of Azure AD app with principal access to subscription so it could be used to add function keys during deployment'
  exit
fi

set -euo pipefail


# Initialize parameters specified from command lin
while getopts ":b:g:n:l:i:a:o:s:u:t:" arg; do
	case "${arg}" in
		b)
			subscriptionId=${OPTARG}
			;;
		g)
			resourceGroupName=${OPTARG}
			;;
		i)
			feedbotId=${OPTARG}
			;;
		n)
			feedbotName=${OPTARG}
			;;
		a)
			appId=${OPTARG}
			;;
		o)
			origin=${OPTARG}
			;;	
		s)
			appSecret=${OPTARG}
			;;
		l)
			botLocale=${OPTARG}
			;;	
		u)
			callbackUrl=${OPTARG}
			;;
		t)
			callbackToken=${OPTARG}
			;;
		esac
done
shift $((OPTIND-1))

# TODO allow setting version by pipeline parameter
if [ ! -z "$callbackUrl" ] && [ ! -z "$callbackToken" ]; then
	declare feedbotVersion="v1.7/v1.7.6"
else
	declare feedbotVersion="v1.5.67"
fi

#Prompt for parameters is some required parameters are missing
if [[ -z "$subscriptionId" ]]; then
	echo "Your subscription ID can be looked up with the CLI using: az account show --out json "
	echo "Enter your subscription ID:"
	read subscriptionId
	[[ "${subscriptionId:?}" ]]
fi

if [[ -z "$deploymentName" ]]; then
	echo "Enter a name for this deployment:"
	read deploymentName
fi

if [[ -z "$feedbotId" ]]; then
	echo "Enter ID for the feedbot without \"feedbot-\" prefix, use only [A-Za-z0-9] (for example \"skodaauto-trainee\"):"
	read feedbotId
fi

if [[ "${feedbotId:0:8}" == "feedbot-" ]]; then
	echo "Feedbot ID should not be prefixed by \"feedbot-\"."
	exit 1
fi

feedbotId="feedbot-${feedbotId}"

if [[ -z "$feedbotName" ]]; then
	echo "Enter a name for the feedbot (for example \"Feedyou chatbot\"):"
	read feedbotName
fi

if [[ -z "$appSecret" ]]; then
	echo "No appSecret -s param found, generating one"
	appSecret=`openssl rand -base64 12`
fi

if [[ -z "$appId" ]]; then
	echo "No appId -a param found, creating AD app"
	appInfo=`az ad app create --display-name $feedbotId --password $appSecret --available-to-other-tenants true --identifier-uris https://$feedbotId.administratorfeedyou.onmicrosoft.com`
	appId=`echo "$appInfo" | grep '"appId":'`
	appId=${appId:12:36}  # TODO refactor to jq
fi

if [[ -z "$resourceGroupName" ]]; then
	echo "No resourceGroupName (-g) param set, using \"${feedbotId}\"."
	resourceGroupName=${feedbotId}
fi

if [[ -z "$resourceGroupLocation" ]]; then
	echo "If creating a *new* resource group, you need to set a location "
	echo "You can lookup locations with the CLI using: az account list-locations "
	
	echo "Enter resource group location:"
	read resourceGroupLocation
fi

templateFilePath="template.json"
if [ ! -f "$templateFilePath" ]; then
	echo "$templateFilePath not found"
	exit 1
fi

parametersFilePath="parameters.json"
if [ ! -f "$parametersFilePath" ]; then
	echo "$parametersFilePath not found"
	exit 1
fi

if [ -z "$subscriptionId" ] || [ -z "$resourceGroupName" ] || [ -z "$deploymentName" ]; then
	echo "Either one of subscriptionId, resourceGroupName, deploymentName is empty"
	usage
fi

if [ -z "$feedbotId" ] || [ -z "$feedbotName" ]; then
	echo "Either one of feedbotId, feedbotName is empty"
	usage
fi

if [ -z "$appId" ] || [ -z "$appSecret" ]; then
	echo "Either one of appId, appSecret is empty"
	usage
fi

# login to azure using your credentials
az account show 1> /dev/null

if [ $? != 0 ];
then
	az login
fi

# set the default subscription id
az account set --subscription $subscriptionId

set +e

#Check for existing RG
az group show -g $resourceGroupName 1> /dev/null

if [ $? != 0 ]; then
	echo "Resource group with name" $resourceGroupName "could not be found. Creating new resource group.."
	set +e	
	az group create --name $resourceGroupName --location $resourceGroupLocation 1> /dev/null
else
	echo "Using existing resource group..."
fi

# generate storage account name
declare storageAccountName=`echo ${feedbotId} | sed -e 's/[^a-zA-Z0-9]//g'`
storageAccountName=${storageAccountName:0:20}`seq 1000 9999 | sort -R | head -n 1`  # TODO refactor to jq
echo "Generated storage account name: $storageAccountName"
set -e

# start deployment
echo "Starting deployment..."
(
	set -x
	# https://docs.microsoft.com/en-us/cli/azure/deployment?view=azure-cli-latest#az-deployment-validate
	az group deployment create --name "$deploymentName" --resource-group "$resourceGroupName" --template-file "$templateFilePath" --parameters "@${parametersFilePath}" --parameters "{\"botId\": {\"value\":\"${feedbotId}\"}, \"botName\": {\"value\":\"${feedbotName}\"}, \"siteName\": {\"value\":\"${feedbotId}\"}, \"appId\": {\"value\":\"${appId}\"}, \"appSecret\": {\"value\":\"${appSecret}\"}, \"storageAccountName\": {\"value\":\"${storageAccountName}\"}, \"botLocale\": {\"value\":\"${botLocale}\"}}" 1> deployment.log
)

if [ $?  == 0 ];
 then
	echo "Template has been successfully deployed"
fi

az functionapp config appsettings set --name $feedbotId --resource-group $resourceGroupName --settings WEBSITE_RUN_FROM_ZIP="https://feedyou.blob.core.windows.net/feedbot-azure-function/${feedbotVersion}.zip?st=2018-08-15T12%3A31%3A33Z&se=2048-08-16T12%3A31%3A00Z&sp=rl&sv=2017-04-17&sr=c&sig=WP9QHjymZkHItcZDwnQXNrMRfrx%2FxXHUiTNBiOQ4nBI%3D" 1> /dev/null

# create upload+public blob containers
az storage container create --name upload --public-acces blob --connection-string `az storage account show-connection-string --resource-group ${resourceGroupName} --name ${storageAccountName} --output tsv` 1> /dev/null
az storage container create --name public --public-acces blob --connection-string `az storage account show-connection-string --resource-group ${resourceGroupName} --name ${storageAccountName} --output tsv` 1> /dev/null

# generate DL key and put into azureWebJobsBotFrameworkDirectLineSecret setting
declare dlSecrets=`az bot directline create --site-name deploy --name $feedbotId --resource-group $resourceGroupName`
proactiveDlSecret=`echo "$dlSecrets" | jq -r .properties.properties.sites[0].key`
az functionapp config appsettings set --name $feedbotId --resource-group $resourceGroupName --settings AzureWebJobsBotFrameworkDirectLineSecret="${proactiveDlSecret}" 1> /dev/null

# generate directline key for designer
declare designerDlSecret=`echo "$dlSecrets" | jq -r .properties.properties.sites[0].key2`

# create new function host key keys
designerHostKey=`./function-host-key.sh $resourceGroupName $feedbotId designer`
botHostKey=`./function-host-key.sh $resourceGroupName $feedbotId bot` # TODO set messages fn only instead of whole host
designerEndpoint="https://$feedbotId.azurewebsites.net/api?code=$designerHostKey"
botEndpoint="https://$feedbotId.azurewebsites.net/api/messages?code=$botHostKey"

# update bot endpoint which is preset to NYI and its wrongly fixed only after first bot blade visit in the portal
az bot update -n $feedbotId -g $resourceGroupName --endpoint "$botEndpoint" 1> /dev/null

# set additional CORS if defined
if [[ ! -z "$origin" ]]; then
	echo "Additional origin URL $origin found in -o arg, setting"
	az webapp cors add -n $feedbotId -g $resourceGroupName --allowed-origins "$origin"
fi

previewUrl="https://feedyou.blob.core.windows.net/webchat/preview.html?id=$feedbotId&name=$feedbotName&key=$designerDlSecret#feedbot-test-mode"

echo "Put these values into bot config in Feedbot Designer:"
echo "  - \"Direct Line Secret\": $designerDlSecret"
echo "  - \"Preview URL\": $previewUrl"
echo "  - \"API URL\": $designerEndpoint"

if [ ! -z "$callbackUrl" ] && [ ! -z "$callbackToken" ]; then
	echo "Sending success callback back to the server..."
	curl -v -G --request PATCH "${callbackUrl}" --data-urlencode "directLineSecret=$designerDlSecret" --data-urlencode "previewUrl=$previewUrl" --data-urlencode "apiUrl=$designerEndpoint" --header "Authorization: Bearer $callbackToken"
else
	echo "Skipping sending success callback back to the server bacause one of parameters callbackUrl or callbackToken is not set (${callbackUrl} ${callbackToken})"
fi
