const jimp = require('jimp')
const azure = require('azure-storage')
var streamifier = require('streamifier')

var createLogger = require('../utils/logger')
const logger = createLogger('azure-function:file-upload-service')

const DEFAULT_THUMBNAIL_HEIGHT = 300
const CONTAINER = 'upload'

let accountName = ''

function list(context, req) {
  // get all blobs from upload/originals/
  const list = type => {
    const blobService = azure.createBlobService(process.env.AzureWebJobsStorage)
    return new Promise((resolve, reject) => {
      blobService.listBlobsSegmentedWithPrefix(CONTAINER, type, null, { delimiter: '' }, function(
        err,
        result
      ) {
        if (err) {
          logger.error("Couldn't list blobs " + err)
          reject(err)
        } else {
          logger.notice('Blobs:')
          resolve(result.entries)
        }
      })
    })
  }
  list('thumbnails').then(thumbnails => {
    list('originals')
      .then(originals => {
        let files = originals.map(original => {
          const filename = original.name.match(/\/.*$/g)[0].substr(1)
          const thumbnail = thumbnails.find(
            thumbnail => thumbnail.name === original.name.replace('originals/', 'thumbnails/')
          )
          return {
            filename: filename,
            original: {
              url: accountName + 'upload/originals/' + filename
            },
            thumbnail: thumbnail ? { url: accountName + 'upload/thumbnails/' + filename } : null
          }
        })

        context.res = {
          status: 200,
          body: files,
          headers: {
            'Content-Type': 'application/json'
          }
        }
        context.done()
      })
      .catch(err => context.done())
  })
}

function upload(context, req) {
  const thumbnailHeight =
    context.req.params.thumbnail && context.req.params.thumbnailHeight
      ? Number(context.req.params.thumbnailHeight)
      : DEFAULT_THUMBNAIL_HEIGHT

  const image = decodeBase64Image(context, req.body)

  if (context.req.params.filename.includes('.')) {
    // filename contains an extention - remove it.
    const extention = context.req.params.filename.match(/\.(.*)$/g)
    context.req.params.filename = context.req.params.filename.replace(extention, '')
  } else {
    logger.warning(
      'BE CAREFUL, file name does not contain an extention - default format is undefined. This might cause problems opening the file of type "File"'
    )
  }

  let filename = context.req.params.filename
  if (image.dataType.includes('image')) {
    // if data type of the input base64 data is image, then append extention format to filename
    const extention = image.dataType.match(/\/(.*)$/g)[0].substr(1)
    filename = filename + '.' + extention
  }

  let response = {
    // TODO remove file extension from end if name property
    name: context.req.params.filename,
    filename: filename,
    original: { url: image.originalUrl }
  }

  logger.log(
    `POST details:\nPOST URL: %s\nFilename: %s\nFiletype: %s\nEncodingType: %s`,
    context.req.url,
    image.filename,
    image.dataType,
    image.type
  )

  // do not resize non-images
  if (!image.dataType.includes('image') || image.dataType === 'image/gif') {
    logger.log('Non-image detected, do not resize')

    context.res = {
      status: 200,
      body: response
    }
    logger.log('Sending response: ' + JSON.stringify(context.res))

    uploadBlockBlob(CONTAINER, image.data, image.dataType, 'originals/' + image.filename, err => {
      err && console.error(err)
      context.done()
    })
  } else {
    context.res = {
      status: 200,
      body: Object.assign(response, { thumbnail: { url: image.thumbnailUrl } })
    }
    logger.log('Sending response: ' + JSON.stringify(context.res))

    resizeBase64Image(context, image.data, thumbnailHeight)
      .then(resized => {
        logger.log('JIMP resize done')
        uploadBlockBlob(
          CONTAINER,
          image.data,
          image.dataType,
          'originals/' + image.filename,
          err => {
            err && console.error(err)
            uploadBlockBlob(
              CONTAINER,
              resized,
              image.dataType,
              'thumbnails/' + image.filename,
              err => {
                err && console.error(err)
                context.done()
              }
            )
          }
        )
      })
      .catch(err => {
        console.error('JIMP resize error', err)
        context.res = {
          status: 500,
          body: { message: err && err.message }
        }
        context.done()
      })
  }
}

function remove(context, req) {
  if (context && context.req && context.req.params && context.req.params.filename) {
    const filename = context.req.params.filename
    const blobOriginal = 'originals/' + filename
    const blobThumbnail = 'thumbnails/' + filename
    const blobService = azure.createBlobService(process.env.AzureWebJobsStorage)
    blobService.deleteBlobIfExists(CONTAINER, blobOriginal, (error, result, response) => {
      logger.warning(JSON.stringify(response, undefined, ' '))
      if (error || response.statusCode !== 202) {
        // if statusCode starts with 2
        error ? logger.error(error) : logger.error(response.body)
        context.res = {
          status: response.statusCode,
          body: 'Could not delete ' + filename + '\n' + response.body || error
        }
        context.done()
      } else {
        blobService.deleteBlobIfExists(CONTAINER, blobThumbnail, error => {
          error && logger.error(error)
          context.res = {
            status: 202,
            body: 'Successfully deleted ' + filename
          }
          context.done()
        })
      }
    })
  } else {
    context.res = {
      status: 400,
      body: 'filename is undefined'
    }
    context.done('filename is undefined')
  }
}

function init() {
  if (process.env.AzureWebJobsStorage) {
    if (process.env.BlobAccountUrl) {
      accountName = process.env.BlobAccountUrl
      logger.info('USING BlobAccountUrl ENV VALUE: ' + accountName)
    } else {
      if (process.env.AzureWebJobsStorage === 'UseDevelopmentStorage=true') {
        accountName = process.env.AzureWebJobsStorage
        logger.error(
          'BlobAccountUrl is not set and AzureWebJobsStorage is set to development mode.'
        )
      } else {
        accountName = process.env.AzureWebJobsStorage.match(/AccountName=([^;]*);/)[1]
        logger.info('AZURE WEB JOBS STORAGE ACCOUNT NAME: ' + accountName)
        accountName = 'https://' + accountName + '.blob.core.windows.net/'
      }
    }
  } else {
    logger.error('AzureWebJobsStorage is undefined')
  }
  if (!accountName) {
    logger.error('accountName is undefined!')
  }
}

module.exports = {
  init,
  list,
  upload,
  remove
}

function uploadBlockBlob(container, data, dataType, filename, callback) {
  const options = { contentSettings: { contentType: dataType } }
  const blobService = azure.createBlobService(process.env.AzureWebJobsStorage)
  const stream = streamifier.createReadStream(data)
  stream.pipe(blobService.createWriteStreamToBlockBlob(container, filename, options, callback))
}

function resizeBase64Image(context, buffer, height) {
  return jimp.read(buffer).then(img => {
    if (height >= img.bitmap.height) {
      logger.log(
        `Requested height of ${height}px is bigger than original height of the source image (${
          img.bitmap.height
        }px): skipping resizing function, returning original image as a thumbnail`
      )
      return Promise.resolve(buffer)
    } else {
      return new Promise((resolve, reject) => {
        img.resize(jimp.AUTO, height).getBuffer(jimp.AUTO, (err, buffer) => {
          if (err) {
            reject(err)
          } else {
            resolve(buffer)
          }
        })
      })
    }
  })
}

function decodeBase64Image(context, data) {
  var matches = []
  matches.push(data.match(/data:([A-Za-z-+\/]+)/g))
  matches.push(data.match(/base64/g))
  matches.push(
    data
      .match(/,(.*)/g)
      .toString()
      .substr(1)
  )

  var response = {}

  if (matches.length !== 3) {
    logger.log(
      '\n\nERROR CASE: file-upload/decodeBase64Image(): matches.length equals ' +
        matches.length +
        ' instead of 3.\n\n'
    )
    return
  }

  response.dataType = matches[0].toString().slice(5) // data:image/jpeg => image/jpeg
  response.type = matches[1].toString() // base64
  response.data = Buffer.from(matches[2], response.type) //  actual base64 code
  response.filename = context.req.params.filename

  response.originalUrl = accountName + 'upload/originals/' + context.req.params.filename
  response.thumbnailUrl = accountName + 'upload/thumbnails/' + context.req.params.filename

  return response
}
