'use strict'

const path = require('path')
const express = require('express')
const azureFunctionExpress = require('azure-function-express')

// we use ts-node to run "messages" function during development, so we need to import source src/*.ts files instead of dist/*.js
// (use eval to prevent funcpack's webpack execution from unnecessasry src folder require)
const nodeSdk =
  process.env.NODE_ENV === 'development'
    ? eval('require')('@dvorakjan/feedbot-node-sdk/src')
    : require('@dvorakjan/feedbot-node-sdk/dist/node-sdk/src')

const builder = nodeSdk.Builder
const builder_azure = nodeSdk.BuilderAzure
const Feedbot = nodeSdk.Feedbot

const createLogger = require('../utils/logger')
const logger = createLogger('azure-function:messages')

// can use .env file during development
require('dotenv').config()

const connector = new builder.ChatConnector({
  appId: process.env['MicrosoftAppId'],
  appPassword: process.env['MicrosoftAppPassword'],
  openIdMetadata: process.env['BotOpenIdMetadata']
})

// feedbot instance creates bot using botbuilder on its own and configures it using given botId
const messagesPath = __dirname.endsWith('.funcpack')
  ? path.join(__dirname, '..', 'messages')
  : __dirname

const feedbot = new Feedbot(connector, messagesPath)

// send traces (console.log) and exceptions to Application Insights
// (Live Metrics Stream doesn't work with WebApp SDK)
if (process.env['BotDevAppInsightsKey']) {
  feedbot.initAppInsights(process.env['BotDevAppInsightsKey'])
}

// do not let user choose locale by himself
feedbot.forceLocale(process.env['BotLocale'])

// save user state into Azure Table storage
if (process.env['CosmosDocumentDbHost'] && process.env['CosmosDocumentDbKey']) {
  feedbot.setCosmosStateStorage(
    process.env['CosmosDocumentDbHost'],
    process.env['CosmosDocumentDbKey'],
    process.env['CosmosDocumentDbDatabase']
  )
} else if (process.env['AzureWebJobsStorage']) {
  feedbot.setStateStorage('StateData', process.env['AzureWebJobsStorage'])
} else {
  feedbot.setMemoryStateStorage()
}

/* ------ This is the place for bot customizations ------ */

/* ------------------------------------------------------ */

const app = express()
feedbot.registerRoutes(app)

if (process.env.NODE_ENV === 'development') {
  // during development, we need to start express manually for the given port
  feedbot.init(err => {
    if (err) {
      logger.error('INIT ERROR ' + err)
    } else {
      const port = process.env['port'] || process.env['PORT'] || 7071
      app.listen(port, () => logger.notice(`EXPRESS LISTENING ON PORT :${port}...`))
    }
  })
} else {
  let isFeedbotReady = false
  let onReadyCallbacks = []

  // init feedbot during function host startup and call all requests saved during initialization when done
  feedbot.init(err => {
    err && console.error('INIT ERROR', err)

    isFeedbotReady = true
    onReadyCallbacks.map(callback => callback())
  })

  // in Azure Function environment, we need to wrap express app using azure-function-express
  const handle = azureFunctionExpress.createHandler(app)
  module.exports = (context, request) => {
    console.log = context.log
    console.error = context.log.error || context.log
    console.warn = context.log.warn || context.log
    console.trace = context.log || context.log
    console.info = context.log.info || context.log

    if (isFeedbotReady) {
      handle(context, request)
    } else {
      console.log('FEEDBOT IS NOT READY YET, ADDING REQUEST TO CALLBACK QUEUE')
      onReadyCallbacks.push(() => handle(context, request))
    }
  }
}
